#!/bin/sh

#
# DaVinci Resolve 12.5.5 Debian package creator
# Version 3
# By Daniel Tufvesson 2017
#

check_command() {
    if [ ! -x "$(command -v $1)" ]; then
	echo "Sorry. Need '$1' to continue."
	exit 1
    fi
}

copy_object() {
    echo "[COPY] $1 [TO] $2"
    cp -rp "$1" "$2"
}

create_directory() {
    if [ -e "$1" ] ; then
	echo "[DIRECTORY EXISTS] $1"
    else
	echo "[CREATE DIRECTORY] $1"
	mkdir -p "$1"
    fi
}

make_executable() {
    if [ -e "$1" ] ; then
	echo "[MAKE EXECUTABLE] $1"
	chmod a+x "$1"
    else
	echo "[MISSING] $1"
    fi
}
extract_gz() {
    check_command gunzip
    if [ -f "$1" ] ; then
	echo "[EXTRACT] $1"
	gunzip -f "$1"
	if [ -e "$1" ] ; then
	    rm "$1"
	fi
    else
	echo "[MISSING] $1"
    fi
}

extract_tgz() {
    check_command tar
    if [ -f "$1" ] ; then
	echo "[EXTRACT] $1"
	tar zxf "$1" -C "$(dirname $1)"
	if [ -e "$1" ] ; then
	    rm "$1"
	fi
    else
	echo "[MISSING] $1"
    fi
}

#
# Script begin here
#
if [ $(id -u) -ne 0 ] ; then
    echo "Sorry. Must be root do this."
    exit 1
fi

# Setup variables
INSTALLER_ARCHIVE=DaVinci_Resolve_12.5.5_Linux.sh
UNPACK_DIR=./unpack
DEB_NAME=davinci-resolve
DEB_VERSION=12.5.5-3
DEB_DIR=./${DEB_NAME}_${DEB_VERSION}_amd64
RESOLVE_INSTALL_DIR=${DEB_DIR}/opt/resolve

# Extract DaVinci Resolve Archive
if [ ! -e "${INSTALLER_ARCHIVE}" ] ; then
    echo "Sorry. Need '${INSTALLER_ARCHIVE}' to continue."
    exit 1
fi
create_directory ${UNPACK_DIR}
sh ${INSTALLER_ARCHIVE} --noexec --keep --target ${UNPACK_DIR}

# Create destination directories
create_directory ${DEB_DIR}
create_directory ${RESOLVE_INSTALL_DIR}

# Create directories
create_directory ${RESOLVE_INSTALL_DIR}/docs
create_directory ${RESOLVE_INSTALL_DIR}/bin
create_directory ${RESOLVE_INSTALL_DIR}/graphics
create_directory ${RESOLVE_INSTALL_DIR}/scripts
create_directory ${RESOLVE_INSTALL_DIR}/configs
create_directory ${RESOLVE_INSTALL_DIR}/Media
create_directory ${RESOLVE_INSTALL_DIR}/"Resolve Disk Database"
create_directory ${RESOLVE_INSTALL_DIR}/.LUT
create_directory ${RESOLVE_INSTALL_DIR}/logs

# Copy common files
copy_object ${UNPACK_DIR}/DaVinci_Resolve_Manual.pdf ${RESOLVE_INSTALL_DIR}/docs
copy_object ${UNPACK_DIR}/Developer ${RESOLVE_INSTALL_DIR}
copy_object ${UNPACK_DIR}/LUT ${RESOLVE_INSTALL_DIR}
copy_object ${UNPACK_DIR}/UI_Resource/ ${RESOLVE_INSTALL_DIR}
copy_object ${UNPACK_DIR}/libs ${RESOLVE_INSTALL_DIR}
copy_object ${UNPACK_DIR}/panels/ ${RESOLVE_INSTALL_DIR}
copy_object ${UNPACK_DIR}/plugins/ ${RESOLVE_INSTALL_DIR}

# Copy standalone files
copy_object ${UNPACK_DIR}/resolve ${RESOLVE_INSTALL_DIR}/bin/
copy_object ${UNPACK_DIR}/rsf/ShowDpxHeader ${RESOLVE_INSTALL_DIR}/bin/
copy_object ${UNPACK_DIR}/rsf/TestIO ${RESOLVE_INSTALL_DIR}/bin/
copy_object ${UNPACK_DIR}/rsf/deviceQuery ${RESOLVE_INSTALL_DIR}/bin/
copy_object ${UNPACK_DIR}/rsf/bandwidthTest ${RESOLVE_INSTALL_DIR}/bin/
copy_object ${UNPACK_DIR}/rsf/oclDeviceQuery ${RESOLVE_INSTALL_DIR}/bin/
copy_object ${UNPACK_DIR}/rsf/oclBandwidthTest ${RESOLVE_INSTALL_DIR}/bin/
copy_object ${UNPACK_DIR}/rsf/DPDecoder ${RESOLVE_INSTALL_DIR}/bin
copy_object ${UNPACK_DIR}/rsf/bmdpaneld ${RESOLVE_INSTALL_DIR}/bin
copy_object ${UNPACK_DIR}/rsf/BMDPanelFirmware ${RESOLVE_INSTALL_DIR}/bin
copy_object ${UNPACK_DIR}/rsf/Control ${RESOLVE_INSTALL_DIR}/
copy_object ${UNPACK_DIR}/rsf/qt.conf ${RESOLVE_INSTALL_DIR}/bin
copy_object ${UNPACK_DIR}/rsf/DV_Resolve.png ${RESOLVE_INSTALL_DIR}/graphics
copy_object ${UNPACK_DIR}/rsf/default-config-linux.dat ${RESOLVE_INSTALL_DIR}/configs/config.dat

# Extract archives
extract_gz ${RESOLVE_INSTALL_DIR}/LUT/trim_lut0.dpx.gz
extract_tgz ${RESOLVE_INSTALL_DIR}/libs/aaf_runtime_libs_x86_64.tgz
extract_tgz ${RESOLVE_INSTALL_DIR}/libs/easydcp-rt-linux-x86_64.tgz
extract_tgz ${RESOLVE_INSTALL_DIR}/libs/ffmpeg-rt-linux-x86_64.tgz
extract_tgz ${RESOLVE_INSTALL_DIR}/libs/graphviz-rt-linux-x86_64.tgz
extract_tgz ${RESOLVE_INSTALL_DIR}/libs/log4cxx_runtime_libs_x64.tgz
extract_tgz ${RESOLVE_INSTALL_DIR}/libs/mxf-rt-linux-x86_64.tgz
extract_tgz ${RESOLVE_INSTALL_DIR}/libs/sonyrawdev-rt-linux-x86_64.tgz
extract_tgz ${RESOLVE_INSTALL_DIR}/libs/smdk-rt-linux-x86_64.tgz
extract_tgz ${RESOLVE_INSTALL_DIR}/libs/sstp-rt-linux-x86_64.tgz
extract_tgz ${RESOLVE_INSTALL_DIR}/libs/sxavc-rt-linux-x86_64.tgz

# Fix user rights
chown -R root:root ${RESOLVE_INSTALL_DIR}

# Create desktop shortcut
create_directory ${DEB_DIR}/usr/share/applications/
cat > ${DEB_DIR}/usr/share/applications/DaVinci\ Resolve.desktop <<EOF
[Desktop Entry]
Version=${DEB_VERSION}
Encoding=UTF-8
Exec=/opt/resolve/bin/resolve
Icon=/opt/resolve/graphics/DV_Resolve.png
MimeType=
Name=DaVinci Resolve
Terminal=false
Type=Application
Categories=AudioVideo
EOF

# Create Debian package
create_directory ${DEB_DIR}/DEBIAN
cat > ${DEB_DIR}/DEBIAN/control <<EOF
Package: ${DEB_NAME}
Version: ${DEB_VERSION}
Architecture: amd64
Maintainer: Blackmagic Design
Description: DaVinci Resolve made from ${INSTALLER_ARCHIVE}
EOF
cat > ${DEB_DIR}/DEBIAN/conffiles <<EOF
/opt/resolve/configs/config.dat
EOF
cat > ${DEB_DIR}/DEBIAN/postinst <<EOF
#!/bin/sh
set -x
test -e /usr/lib/x86_64-linux-gnu/libcrypto.so.1.0.0 && ln -s /usr/lib/x86_64-linux-gnu/libcrypto.so.1.0.0 /opt/resolve/libs/libcrypto.so.10
test -e /lib/x86_64-linux-gnu/libcrypto.so.1.0.0 && ln -s /lib/x86_64-linux-gnu/libcrypto.so.1.0.0 /opt/resolve/libs/libcrypto.so.10
test -e /usr/lib/x86_64-linux-gnu/libssl.so.1.0.0 && ln -s /usr/lib/x86_64-linux-gnu/libssl.so.1.0.0 /opt/resolve/libs/libssl.so.10
test -e /lib/x86_64-linux-gnu/libssl.so.1.0.0 && ln -s /lib/x86_64-linux-gnu/libssl.so.1.0.0 /opt/resolve/libs/libssl.so.10
test -e /usr/lib/x86_64-linux-gnu/libgstbase-1.0.so.0 && ln -s /usr/lib/x86_64-linux-gnu/libgstbase-1.0.so.0 /opt/resolve/libs/libgstbase-0.10.so.0
test -e /lib/x86_64-linux-gnu/libgstbase-1.0.so.0 && ln -s /lib/x86_64-linux-gnu/libgstbase-1.0.so.0 /opt/resolve/libs/libgstbase-0.10.so.0
test -e /usr/lib/x86_64-linux-gnu/libgstreamer-1.0.so.0 && ln -s /usr/lib/x86_64-linux-gnu/libgstreamer-1.0.so.0 /opt/resolve/libs/libgstreamer-0.10.so.0
test -e /lib/x86_64-linux-gnu/libgstreamer-1.0.so.0 && ln -s /lib/x86_64-linux-gnu/libgstreamer-1.0.so.0 /opt/resolve/libs/libgstreamer-0.10.so.0
chmod -R a+rw /opt/resolve/configs
chmod -R a+rw /opt/resolve/Media
chmod -R a+rw /opt/resolve/"Resolve Disk Database"
chmod -R a+rw /opt/resolve/.LUT
chmod -R a+rw /opt/resolve/logs
EOF
make_executable ${DEB_DIR}/DEBIAN/postinst
cat > ${DEB_DIR}/DEBIAN/prerm <<EOF
#!/bin/sh
set -x
rm -f /opt/resolve/libs/libcrypto.so.10
rm -f /opt/resolve/libs/libssl.so.10
rm -f /opt/resolve/libs/libgstbase-0.10.so.0
rm -f /opt/resolve/libs/libgstreamer-0.10.so.0
EOF
make_executable ${DEB_DIR}/DEBIAN/prerm
check_command dpkg-deb
dpkg-deb -b ${DEB_DIR} ${DEB_DIR}.deb

# Clean up
rm -rf ${DEB_DIR}
rm -rf ${UNPACK_DIR}
